﻿using Spectre.Console;

namespace VRC_Console;

public partial class OSCCollar
{
    private const byte CoordinateSystemSize = 16;
    private Task SetupConsoleOutput()
    {
        Layout layout = new Layout().SplitRows(
            new Layout("Top"),
            new Layout("Middle").SplitColumns(
                new Layout("Config-Values"),
                new Layout("Variable-Values"),
                new Layout("Position")
            ),
            new Layout("Bottom"));
        if(OperatingSystem.IsWindows())
            Console.WindowWidth += 20;
        Table configTable = new Table();
        configTable.AddColumn("ConVar");
        configTable.AddColumn("Value");
        configTable.AddRow("IP", this._config.Ip);
        configTable.AddRow("Send-Osc-Port", this._config.PortSend.ToString());
        configTable.AddRow("Receive-Osc-Port", _portReceive.ToString());
        configTable.AddRow("HTTP-Port", this.OscQueryService?.TcpPort.ToString() ?? "");
        configTable.AddRow("CalibrationX", $"{this._config.CalibrationX}");
        configTable.AddRow("CalibrationY", $"{this._config.CalibrationY}");
        configTable.AddRow(Resources.OSCCollar_ConsoleOutput_StretchDeadzone, $"{this._config.WalkStretchDeadzone}");
        configTable.AddRow(Resources.OSCCollar_ConsoleOutput_RunStretch, $"{this._config.RunStretch}");
        
        Table variableTable = new Table();
        variableTable.AddColumn("OscVar");
        variableTable.AddColumn("Value");
        
        layout["Top"].Update(new FigletText(FigletFont.Parse(Resources.alligatorFont), "OSC Collar").LeftJustified());
        layout["Config-Values"].Update(configTable);
        layout["Variable-Values"].Update(variableTable);
        layout["Position"].Update(new Panel(new Text(GetCoordinateSystem())));
        layout["Bottom"].Update(new Text($"{_lastNilMessageSent} AvatarId: {_avatarId} Runtime: {DateTime.Now.Subtract(_programStarted)}"));
        
        variableTable.AddRow("Status", "");
        variableTable.AddRow("GPS 1", "");
        variableTable.AddRow("GPS 2", "");
        variableTable.AddRow("GPS 3", "");
        variableTable.AddRow(Resources.OSCCollar_ConsoleOutput_PositionVector, "");
        variableTable.AddRow(Resources.OSCCollar_ConsoleOutput_LeashStretch, "");
        variableTable.AddRow(Resources.OSCCollar_ConsoleOutput_MovementVector, "");
        variableTable.AddRow("CalibrationX", "");
        variableTable.AddRow("CalibrationY", "");
        
        return AnsiConsole.Live(layout).StartAsync(async displayContext =>
        {
            while (true)
            {
                variableTable.Rows.Update(0, 1, new Text($"{(_allowMoving ? Resources.OSCCollar_ConsoleOutput_StatusEnabled : Resources.OSCCollar_ConsoleOutput_StatusDisabled)}"));
                variableTable.Rows.Update(1, 1, new Text($"{_gps1.Distance:00.00000}"));
                variableTable.Rows.Update(2, 1, new Text($"{_gps2.Distance:00.00000}"));
                variableTable.Rows.Update(3, 1, new Text($"{_gps3.Distance:00.00000}"));
                variableTable.Rows.Update(4, 1, new Text($"{_unitVectorLeash}"));
                variableTable.Rows.Update(5, 1, new Text($"{_leashStretch:0.00000}"));
                variableTable.Rows.Update(6, 1, new Text($"{_movementVector}"));
                variableTable.Rows.Update(7, 1, new Text($"{GetCalibrationValues().Item1}"));
                variableTable.Rows.Update(8, 1, new Text($"{GetCalibrationValues().Item2}"));
                layout["Position"].Update(new Panel(new Text(GetCoordinateSystem())));
                layout["Bottom"].Update(new Text($"{_lastNilMessageSent} AvatarId: {_avatarId} Runtime: {DateTime.Now.Subtract(_programStarted)}"));
                displayContext.Refresh();
                await Task.Delay(100);
            }
        });
    }

    private String GetCoordinateSystem()
    {
        string system = "";
        byte width = CoordinateSystemSize;
        byte height = CoordinateSystemSize / 2;

        int positionX = Convert.ToInt32(Math.Floor(_unitVectorLeash.X * (width / 2.0))) + (width / 2);
        int positionY = Convert.ToInt32(-Math.Floor(_unitVectorLeash.Y * (height / 2.0))) + (height / 2);

        int movementX = Convert.ToInt32(Math.Floor(_movementVector.X * (width / 2.0))) + (width / 2);
        int movementY = Convert.ToInt32(-Math.Floor(_movementVector.Y * (height / 2.0))) + (height / 2);

        for (int y = 0; y <= height; y++)
        {
            for (int x = 0; x <= width; x++)
            {

                if (x == movementX && y == movementY)
                {
                    system += "x";
                    continue;
                }

                if (x == positionX && y == positionY)
                {
                    system += "x";
                    continue;
                }

                if (y == height / 2)
                {
                    if (x == width / 2)
                        system += "+";
                    else
                        system += "-";
                }
                else
                {
                    if (x == width / 2)
                        system += "|";
                    else
                        system += " ";
                }
            }

            if (y != height)
                system += "\n";
        }

        return system;
    }
}